package com.bull.cimero.pluginEditor.editors;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PlatformUI;

import org.eclipse.swt.dnd.DropTargetListener;

/**
 * class used to permit parts actualisation.
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class CimeroRegistry {

	/** It is the instance of this class. */
	private static CimeroRegistry currentInstance = null;

	/**
	 * Registred drop Target Listeners.
	 */
	private List<DropTargetListener> dropTargetListenerList =
		new ArrayList<DropTargetListener>();

	/** It is the instance of this class. */
	private List < Tuple > lstTuples = new ArrayList < Tuple >();

	/**
	 * method used to get the active diagramEditor.
	 * @return DiagramEditor : active diagramEditor
	 */
	public final DiagramEditor getActiveEditor() {
		DiagramEditor diag = null;
		try {
			IWorkbenchPage page = PlatformUI.getWorkbench()
			.getActiveWorkbenchWindow().getActivePage();
			diag = (DiagramEditor) page.getActiveEditor();
		} catch (Exception e) {
			diag = null;
		}
		return diag;
	}

	/**
	 * method used to get the diagramEditor associated
	 * with the given filename.
	 * @param name of the file
	 * @return DiagramEditor : wanted diagramEditor
	 */
	public final DiagramEditor getDiagramEditor(final String name) {
		//look for the editor
		Iterator iterator = getLstTuples().iterator();
		while (iterator.hasNext()) {
			Tuple elt = (Tuple) iterator.next();
			String fileName = elt.getFileName();
			if (fileName.compareTo(name) == 0) {
				//found
				return elt.getDiagram();
			}
		}
		return null;
	}

	/**
	 * method used to add a new DiagramEditor to the list.
	 * @param diag to add
	 */
	public final void add(final DiagramEditor diag) {
		//look for an other Editor with the same file
		Tuple tupleToDelete = null;
		Iterator iterator = getLstTuples().iterator();
		while (iterator.hasNext()) {
			Tuple elt = (Tuple) iterator.next();
			String fileName = elt.getDiagram().getFileName();
			if (fileName.compareTo(diag.getFileName()) == 0) {
				tupleToDelete = elt;
			}
		}
		//remove if already exist
		if (tupleToDelete != null) {
			getLstTuples().remove(tupleToDelete);
		}
		//add the new
		Tuple tuple = new Tuple(diag, diag.getFileName());
		getLstTuples().add(tuple);

		// add all registred DropTargetListener to the diagramEditor
		for (Iterator<DropTargetListener> i = dropTargetListenerList.iterator();
			i.hasNext();) {
			DropTargetListener dropTargetListener = i.next();
			diag.addDropTargetListener(dropTargetListener);
		}
	}

	/**
	 * method used to register a new DropTargetListener.
	 * @param dropTargetListener the DropTargetListener to register
	 */
	public final void addDropTargetListener(
			final DropTargetListener dropTargetListener) {
		//System.out.println("Registring a new TargetListener");
		dropTargetListenerList.add(dropTargetListener);
		// Add DropTargetListener to existing DiagramEditors
		for (Iterator<Tuple> i = lstTuples.iterator();
			i.hasNext();) {
			Tuple t = i.next();
			t.getDiagram().addDropTargetListener(dropTargetListener);
		}
	}

	/**
	 * Method used to get all registred DropTargetListener.
	 * @return the dropTargetListenerList
	 */
	final List<DropTargetListener> getDropTargetListenerList() {
		// use the package visibility
		return dropTargetListenerList;
	}

	/**
	 * method used to remove a new DiagramEditor from the list.
	 * @param diag to remove
	 */
	public final void remove(final DiagramEditor diag) {
		Tuple tupleToRemove = null;
		Iterator iterator = getLstTuples().iterator();
		while (iterator.hasNext()) {
			Tuple elt = ((Tuple) iterator.next());
			if (elt.getDiagram() == diag) {
				//we backup this good element
				tupleToRemove = elt;
				break;
			}
		}
		if (tupleToRemove != null) {
			//we delete the element
			getLstTuples().remove(tupleToRemove);
		}
	}

	/**
	 * This function is called to get the instance class.
	 * @return IValidator
	 */
	public static final CimeroRegistry getInstance() {
		if (currentInstance == null) {
			currentInstance = new CimeroRegistry();
		}
		return currentInstance;
	}

	/**
	 * get the list of tuples.
	 * @return Returns the lstTuples.
	 */
	public final List < Tuple > getLstTuples() {
		return lstTuples;
	}

	/**
	 * set the list of tuples.
	 * @param lstTuplesParam The lstTuples to set.
	 */
	public final void setLstTuples(final List < Tuple > lstTuplesParam) {
		this.lstTuples = lstTuplesParam;
	}

	/**
	 * class used to add DiagramEditor AND filename to the list.
	 * diagramEditor knows its filename but it's fastest to store
	 * the filename because it's often use.
	 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
	 */
	public class Tuple {

		/** the diagramEditor instance. */
		private DiagramEditor diagram;
		/** the filename loaded in the diagramEditor. */
		private String fileName;

		/**
		 * default constructor.
		 * @param diag : diag to set
		 * @param name : filename
		 */
		public Tuple(final DiagramEditor diag
				, final String name) {
			setDiagram(diag);
			setFileName(name);
		}

		/**
		 * get the diagramEditor.
		 * @return Returns the diagram.
		 */
		public final DiagramEditor getDiagram() {
			return diagram;
		}

		/**
		 * set the diagramEditor.
		 * @param diagramParam The diagram to set.
		 */
		public final void setDiagram(final DiagramEditor diagramParam) {
			this.diagram = diagramParam;
		}

		/**
		 * get the filename.
		 * @return Returns the fileName.
		 */
		public final String getFileName() {
			return fileName;
		}

		/**
		 * set the filename.
		 * @param fileNameParam The fileName to set.
		 */
		public final void setFileName(final String fileNameParam) {
			this.fileName = fileNameParam;
		}
	}
}