

package com.bull.cimero.pluginEditor.editors.figure;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Vector;
import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.MarginBorder;
import org.eclipse.draw2d.XYLayout;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;

/**
 * The <code>GeneriqueCimeroFigure</code> class is the base
 * class for all the figures representing
 * Cimero, ie those that can have ports with connections to other nodes.
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public abstract class GeneriqueCimeroFigure extends Figure {

	/** The hashtable that containt the connection anchors. */
	private Hashtable connectionAnchors = new Hashtable(HASHTABLECAPABILITY);
	/** The vector that contain all the input port. */
	private Vector inputConnectionAnchors = new Vector(2, 2);
	/** The vector contain all the output port. */
	private Vector outputConnectionAnchors = new Vector(2, 2);
	/** The vector contain all the bottow output port. */
	private Vector outputBottomConnectionAnchors = new Vector(2, 2);
	/** dimensions (width and height) of a port. */
	private static final int PORTDIMENSION = 15;
	/** distance between 2 ports. */
	private static final int PORTSEPARATION = 17;
	/** margin border (tooltip). */
	private static final int TOOLTIPMARGINBORDER = 3;
	/** hashtable capability (connectionAnchors). */
	private static final int HASHTABLECAPABILITY = 7;
	/** validation attribute. */
	private boolean valideDraw = true;

	/**
	 * The default constructor.
	 * It create an XYLayout so the validate function must to be implement
	 */
	public GeneriqueCimeroFigure() {
		XYLayout layout = new XYLayout();
		setLayoutManager(layout);
		setBackgroundColor(ColorConstants.white);
		setOpaque(false);
//		addMouseListener(new MouseListenAction());
	}

	/**
	 * Add an input port and its anchor.
	 * @param portName unique name to refer to the port
	 */
	@SuppressWarnings("unchecked")
	public final void addInput(final String portName) {
		PortFigure	inputPort = new PortFigure();
		add(inputPort);
		PortConnectionAnchor anchor = new PortConnectionAnchor(inputPort);
		getTargetConnectionAnchors().add(anchor);
		connectionAnchors.put(portName, anchor);
	}

	/**
	 * A convenience method for nodes that have a single input port.
	 * It is assigned the default name
	 */
	public final void addDefaultInput() {
		addInput("input");
	}

	/**
	 * Add an output port and its anchor.
	 * @param portName unique name to refer to the port
	 */
	@SuppressWarnings("unchecked")
	public final void addOutput(final String portName) {
		PortFigure outputPort = new PortFigure();
		add(outputPort);

		PortConnectionAnchor anchor = new PortConnectionAnchor(outputPort);
		getSourceConnectionAnchors().add(anchor);
		connectionAnchors.put(portName, anchor);
	}

	/**
	 * Add an output bottom port and its anchor.
	 * @param portName unique name to refer to the port
	 */
	@SuppressWarnings("unchecked")
	public final void addOutputBottom(final String portName) {
		PortFigure outputPort = new PortFigure();
		add(outputPort);

		PortConnectionAnchor anchor = new PortConnectionAnchor(outputPort);
		getSourceBottomConnectionAnchors().add(anchor);
		connectionAnchors.put(portName, anchor);
	}

	/**
	 * Remove an output port and its anchor, given the port name.
	 * @param portName unique name to refer to the port
	 */
	public final void removeOutput(final String portName) {
		PortConnectionAnchor anchor = (PortConnectionAnchor)
		connectionAnchors.get(portName);
		if (anchor != null) {
			connectionAnchors.remove(portName);
			getSourceConnectionAnchors().remove(anchor);
			remove(anchor.getOwner());
		}
	}

	/**
	 * Remove an input port and its anchor, given the port name.
	 * @param portName unique name to refer to the port
	 */
	public final void removeInput(final String portName) {
		PortConnectionAnchor anchor = (PortConnectionAnchor)
		connectionAnchors.get(portName);
		if (anchor != null) {
			connectionAnchors.remove(portName);
			getTargetConnectionAnchors().remove(anchor);
			remove(anchor.getOwner());
		}
	}

	/**
	 * A convenience method for nodes that have a single output port.
	 * It is assigned the default name
	 */
	public final void addDefaultOutput() {
		addOutput("output");
	}

	/**
	 * The comment property of a WorkflowNode is displayed as a tool tip.
	 *  This method is used to set it. Passing
	 * a null or zero-length string will remove the tool tip
	 *
	 * @param comment the comment to use for this node's tool tip
	 */
	public final void setComment(final String comment) {
		Label toolTip = null;

		if (comment != null && comment.length() > 0) {
			toolTip = new Label(comment);
			toolTip.setBorder(new MarginBorder(TOOLTIPMARGINBORDER));
		}

		setToolTip(toolTip);
	}

	/**
	 * Searches for and returns the anchor on this figure
	 * that is closest to the reference point <code>p</code>.
	 * @param p the reference point
	 * @return the anchor on this figure that is closest to <code>p</code>
	 */
	public final ConnectionAnchor connectionAnchorAt(final Point p) {
		ConnectionAnchor closest = null;
		long min = Long.MAX_VALUE;

		Enumeration e = getSourceConnectionAnchors().elements();
		while (e.hasMoreElements()) {
			ConnectionAnchor c = (ConnectionAnchor) e.nextElement();
			Point p2 = c.getLocation(null);
			long d = p.getDistance2(p2);
			if (d < min) {
				min = d;
				closest = c;
			}
		}
		e = getTargetConnectionAnchors().elements();
		while (e.hasMoreElements()) {
			ConnectionAnchor c = (ConnectionAnchor) e.nextElement();
			Point p2 = c.getLocation(null);
			long d = p.getDistance2(p2);
			if (d < min) {
				min = d;
				closest = c;
			}
		}
		return closest;
	}

	/**
	 * returns an anchor given its name.
	 * @param portName name of the anchor
	 * @return the anchor with the name <code>portName</code>
	 */
	public final ConnectionAnchor getConnectionAnchor(final String portName) {
		return (ConnectionAnchor) connectionAnchors.get(portName);
	}

	/**
	 * returns the name of the specified anchor.
	 * @param c the anchor whose name is requested
	 * @return the name of the specifed anchor
	 */
	public final String getConnectionAnchorName(final ConnectionAnchor c) {
		Enumeration enum2 = connectionAnchors.keys();
		String key;
		while (enum2.hasMoreElements()) {
			key = (String) enum2.nextElement();
			if (connectionAnchors.get(key).equals(c)) {
				return key;
			}
		}
		return null;
	}

	/**
	 * returns the source connection anchor that is closest to
	 * the reference point.
	 * @param p the reference point
	 * @return the closest connection anchor to <code>p</code>
	 */
	public final ConnectionAnchor getSourceConnectionAnchorAt(final Point p) {
		ConnectionAnchor closest = null;
		long min = Long.MAX_VALUE;

		Enumeration e = getSourceConnectionAnchors().elements();
		while (e.hasMoreElements()) {
			ConnectionAnchor c = (ConnectionAnchor) e.nextElement();
			Point p2 = c.getLocation(null);
			long d = p.getDistance2(p2);
			if (d < min) {
				min = d;
				closest = c;
			}
		}
		return closest;
	}

	/**
	 * returns all the source connection anchors on this node figure.
	 * @return a vector of all the source connection anchors for this
	 * figure.
	 */
	public final Vector getSourceConnectionAnchors() {
		return outputConnectionAnchors;
	}

	/**
	 * returns all the source bottom connection anchors on this node figure.
	 * @return a vector of all the source bottom connection anchors for this
	 * figure.
	 */
	public final Vector getSourceBottomConnectionAnchors() {
		return outputBottomConnectionAnchors;
	}

	/**
	 * returns the target connection anchor that is closest to the
	 * reference point.
	 * @param p the reference point
	 * @return the closest target connection anchor to <code>p</code>
	 */
	public final ConnectionAnchor getTargetConnectionAnchorAt(final Point p) {
		ConnectionAnchor closest = null;
		long min = Long.MAX_VALUE;

		Enumeration e = getTargetConnectionAnchors().elements();
		while (e.hasMoreElements()) {
			ConnectionAnchor c = (ConnectionAnchor) e.nextElement();
			Point p2 = c.getLocation(null);
			long d = p.getDistance2(p2);
			if (d < min) {
				min = d;
				closest = c;
			}
		}
		return closest;
	}

	/**
	 * returns all the target connection anchors on this node figure.
	 * @return a vector of all the target connection anchors for this figure
	 */
	public final Vector getTargetConnectionAnchors() {
		return inputConnectionAnchors;
	}

	/**
	 * Returns the name of the specified anchor.
	 *
	 * @param anchor the connectio anchor whose name is requested
	 * @return the anchor's name
	 */
	public final String getNameForAnchor(
			final ConnectionAnchor anchor) {
		Iterator it = connectionAnchors.keySet().iterator();
		String name;

		while (it.hasNext()) {
			name = (String) it.next();

			if (anchor.equals(connectionAnchors.get(name))) {
				return name;
			}
		}
		return null;
	}

	/**
	 * layout method is overwritten to arrange the input
	 * and output ports around the edges of the node figures.
	 */
	public void validate() {
		//we invalidate current figure
		invalidate();
		//
		Iterator it;
		ConnectionAnchor port;
		int y, x;
		//
		// the target source anchor
		//
		x = 0;
		y = getFigureDimension().height / 2;
		it = getTargetConnectionAnchors().iterator();
		while (it.hasNext()) {
			port = (ConnectionAnchor) it.next();

			getLayoutManager().setConstraint(port.getOwner(),
					new Rectangle(x, y, -1, -1));

			y += PORTDIMENSION + PORTSEPARATION;
		}
		//
		// the source output anchor (left)
		//
		x = getFigureDimension().width - PORTDIMENSION;
		int i = getSourceConnectionAnchors().size();
		if (i == 0) {
			i = 1;
		}
		y = getFigureDimension().height / (i + 1);
		it = getSourceConnectionAnchors().iterator();
		while (it.hasNext()) {
			port = (ConnectionAnchor) it.next();
			getLayoutManager().setConstraint(port.getOwner(),
					new Rectangle(x, y, -1, -1));

			y += getFigureDimension().height / (i + 1);
		}
		//
		// the bottom source anchor
		//
		x = getFigureDimension().width / 2 - PORTDIMENSION;
		i = getSourceBottomConnectionAnchors().size();
		if (i == 0) {
			i = 1;
		}
		y = getFigureDimension().height;
		it = getSourceBottomConnectionAnchors().iterator();
		while (it.hasNext()) {
			port = (ConnectionAnchor) it.next();
			getLayoutManager().setConstraint(port.getOwner(),
					new Rectangle(x, y, -1, -1));

			y += getFigureDimension().height / (i + 1);
		}
		super.validate();
	}

	/**
	 * method to test using local coordinates..
	 * @return boolean
	 */
	protected final boolean useLocalCoordinates() {
		return true;
	}

	/**
	 * method to set the name.
	 * @param nameParam : the name
	 */
	public void setName(final String nameParam) {
	}

	/**
	 * method toget the figure dimension.
	 * @return Dimension : the dimension
	 */
	public abstract Dimension getFigureDimension();

	/** method to get the connectionAnchors.
	 * @return Returns the connectionAnchors.
	 */
	public final Hashtable getConnectionAnchors() {
		return connectionAnchors;
	}

	/** method to set the connectionAnchors.
	 * @param connectionAnchorsParam The connectionAnchors to set.
	 */
	public final void setConnectionAnchors(
			final Hashtable connectionAnchorsParam) {
		this.connectionAnchors = connectionAnchorsParam;
	}

	/** method to get the inputConnectionAnchors.
	 * @return Returns the inputConnectionAnchors.
	 */
	public final Vector getInputConnectionAnchors() {
		return inputConnectionAnchors;
	}

	/** method to set the inputConnectionAnchors.
	 * @param inputConnectionAnchorsParam The inputConnectionAnchors to set.
	 */
	public final void setInputConnectionAnchors(
			final Vector inputConnectionAnchorsParam) {
		this.inputConnectionAnchors = inputConnectionAnchorsParam;
	}

	/** method to get the outputConnectionAnchors.
	 * @return Returns the outputConnectionAnchors.
	 */
	public final Vector getOutputConnectionAnchors() {
		return outputConnectionAnchors;
	}

	/**
	 * get the validation.
	 * @return Returns the valideDraw.
	 */
	public final boolean isValideDraw() {
		return valideDraw;
	}

	/**
	 * set the validation.
	 * @param valideDrawParam The valideDraw to set.
	 */
	public void setValideDraw(final boolean valideDrawParam) {
		valideDraw = valideDrawParam;
	}

	/**
	 * abstract method to get the path of the current error figure.
	 * @return String : wanted path
	 */
	public abstract String getPathToErrorFigure();

	/**
	 * abstract method to get the path of the current figure.
	 * @return String : wanted path
	 */
	public abstract String getPathToFigure();

	/**
	 * abstract method to get the path of the current error figure.
	 * @param path : path to set
	 */
	public abstract void setPathToErrorFigure(final String path);

	/**
	 * abstract method to get the path of the current figure.
	 * @param path : path to set
	 */
	public abstract void setPathToFigure(final String path);

	/**
	 * abstract method to get the path of the icon of current figure.
	 * @return String : wanted path
	 */
	public abstract String getPathToIcon();

	/**
	 * abstract method to get the path of the icon of current figure.
	 * @param path : path to set
	 */
	public abstract void setPathToIcon(final String path);

}
