package com.bull.cimero.pluginEditor.generator.serviceMixGenerator.xmlGenerator;

import java.util.List;

import com.bull.cimero.pluginEditor.editors.model.Diagram;
import com.bull.cimero.pluginEditor.editors.model.GeneriqueCimeroModel;
import com.bull.cimero.pluginEditor.editors.model.BCModel.BindingComponentModel;
import com.bull.cimero.pluginEditor.editors.model.BCModel.ServiceWeb;
import com.bull.cimero.pluginEditor.editors.model.SEModel.Pipeline;
import com.bull.cimero.pluginEditor.editors.model.SEModel.Routeur;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory.FactoryFactory;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory.GenericServiceMixFactory;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory.lWFactory.GeneriqueLWfactory;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory.lWFactory.SE.RouteurFactory;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory.standardFactory.SE.PipelineFactory;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.IModelServiceMix;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.NameSpaceClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.xmlModel.ContainerClass;
import com.bull.cimero.pluginEditor.utils.log.CimeroLog;

/**
 * This class is the factory of the serviceMix model. It is used
 * with a CimeroModel to generate the servicemix model associated. It will
 * be used to generate the xml configuration package.
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class XmlModelFactory {

	/**The default namespace name.*/
	private static final String DEFAULTNAMESPACENAME = "sm";
	/**The default namespace value.*/
	private static final String DEFAULTNAMESPACEVALUE =
		"http://servicemix.apache.org/config/1.0";
	/**The cimero model.*/
	private Diagram cimeroModel = null;
	/**This namespace is the default namespace using by servicemix.*/
	private NameSpaceClass defaultNamespace;
	/**to determine if the validation must to be do or not.*/
	private boolean isForce;
	/**The namejar using in this model.*/
	private String nameJar;

	/**
	 * This constructor has an parameter.
	 * @param cimeroD : the diagram which will be use to generate the
	 * 	ServiceMix jar model.
	 * @param force : false if the validation must to be do
	 */
	public XmlModelFactory(final Diagram cimeroD, final boolean force) {
		this.cimeroModel = cimeroD;
		this.isForce = force;
	}

	/**
	 * this function is used to generate the servicemix model.
	 * The generation is based on the CimeroModel.
	 * @return the servicemix model
	 */
	public final IModelServiceMix generateModelServiceMix() {
		if (cimeroModel != null) {
			//set the name of the file
			setNameJar(cimeroModel.getCimeroDiagramName());
			//we create the container : the top level.
			ContainerClass container = createContainer();
			//we add all the element to this container
			for (int i = 0; i < cimeroModel.getLstCimeroObject().size(); i++) {
				GeneriqueCimeroModel elt = (GeneriqueCimeroModel)
				cimeroModel.getLstCimeroObject().get(i);
				//we test if the component is valid
				if ((!elt.isValide(false)) && (!isForce)) {
					//not valid
					return null;
				}

				if (elt instanceof Pipeline) {
					addPipeline(elt, container);
				} else if (elt instanceof Routeur) {
					addRouteur(elt, container);
				} else {
					// create the factory
					GenericServiceMixFactory myFact = null;
					try {
						if (elt instanceof BindingComponentModel) {
							BindingComponentModel bcElt =
								(BindingComponentModel) elt;
							myFact = FactoryFactory.createFactory(
								elt, this.defaultNamespace, !bcElt.isInput());
						} else {
							myFact = FactoryFactory.createFactory(
								elt, this.defaultNamespace, false);
						}
					} catch (Exception e) {
						CimeroLog.errorLog("factory creation error", e);
					}

					//call the createXMLModel function
					try {
						container.addActivationSpec(
								myFact.createXMLModel(elt, isForce));
						//add the namespace
						addNamespaceInContainer(
							myFact.getNamespaceComponent().getNameSpaceName(),
							myFact.getNamespaceComponent().getNameSpaceValue(),
							container);
					} catch (Exception e) {
					   CimeroLog.errorLog("exception during XML generation "
							+ "of the factory :" + myFact.getClass().getName(),
							e);
					}
				}
			}
			return container;
		} else {
			return null;
		}
	}

	/**
	 * add in the container a pipeline and http binding component.
	 * @param elt : the pipeline cimero component.
	 * @param container : the container.
	 */
	private void addPipeline(final GeneriqueCimeroModel elt,
			final ContainerClass container) {
		if (elt instanceof Pipeline) {
			Pipeline myPipe = (Pipeline) elt;

			//add the pipeline component
			GenericServiceMixFactory myFact =
				new PipelineFactory(this.defaultNamespace);
			try {
				container.addActivationSpec(
						myFact.createXMLModel(elt, isForce));
				addNamespaceInContainer(
						myFact.getNamespaceComponent().getNameSpaceName(),
						myFact.getNamespaceComponent().getNameSpaceValue(),
						container);
			} catch (Exception e) {
				CimeroLog.errorLog("exception during the pipeline xml"
						+ " model generation" , e);
			}

			//add the pipeline http component.
			ServiceWeb wsTmp = new ServiceWeb();
			wsTmp.setInput(true);
			wsTmp.setOutput(false);
			wsTmp.setName(elt.getName()
					+ PipelineFactory.HTTPSUFFIXE);
			wsTmp.setHostNameAdress(myPipe.getLocationURI());
			wsTmp.setSoapRequest(myPipe.isSoapRequest());

			try {
				myFact = FactoryFactory.createFactory(
						wsTmp, this.defaultNamespace, !wsTmp.isInput());
				container.addActivationSpec(
						myFact.createXMLModel(wsTmp, isForce));
				//add the namespace
				addNamespaceInContainer(
					myFact.getNamespaceComponent().getNameSpaceName(),
					myFact.getNamespaceComponent().getNameSpaceValue(),
					container);
			} catch (Exception e) {
				CimeroLog.errorLog("exception during the pipeline xml"
						+ " model generation" , e);
			}

		} else {
			CimeroLog.errorLog("wrong parameter type", new Exception());
		}
	}

	/**
	 * THis function is used to add a namespace in the container.
	 * @param nmsName : the name of the namespace
	 * @param nmsValue : the value of this namespace
	 * @param container : the container
	 */
	private void addNamespaceInContainer(final String nmsName,
			final String nmsValue, final ContainerClass container) {
		//we search if the namespace of this component
		//exist. if not, we add it to the list of namespace.
		if (!namespaceExist(container.getLstNameSpace() , nmsName)) {
			container.addNameSpace(new NameSpaceClass(nmsName , nmsValue));
		}
	}

	/**
	 * This function is used to add a routeur to the container.
	 * @param elt : the cimero routeur instance.
	 * @param container : the container where the routeur must to be add.
	 */
	private void addRouteur(final GeneriqueCimeroModel elt ,
			final ContainerClass container) {
		RouteurFactory myFact = new RouteurFactory(this.defaultNamespace);
		try {
			myFact.setPrefixeJar(getNameJar() + "droolsFile_");
			container.addActivationSpec(myFact.createXMLModel(elt, isForce));
			//add the namespace
			addNamespaceInContainer(RouteurFactory.NAMESPACENAME ,
					RouteurFactory.NAMESPACEVALUE , container);
		} catch (Exception e) {
			CimeroLog.errorLog("exception during the routeur model generation",
					e);
		}
	}

	/**
	 * This function will create the container of the xml configuration file.
	 * @return the container instance.
	 */
	private ContainerClass createContainer() {
		//we create the container
		ContainerClass myConn = new ContainerClass();

		this.defaultNamespace = new NameSpaceClass(DEFAULTNAMESPACENAME ,
				DEFAULTNAMESPACEVALUE);
		//we add the default instance
		myConn.setNameSpaceType(this.defaultNamespace);
		//add the foo namespace for all component.
		addNamespaceInContainer(GeneriqueLWfactory.NAMESPACENAME,
				GeneriqueLWfactory.NAMESPACEVALUE, myConn);
		return myConn;
	}

	/**
	 * This function return true if the namespace passed in parameter
	 * exists in the list of namespace of this container.
	 * @param lst : the list of namespaces.
	 * @param name : the name of the namespace to research.
	 * @return true if the namespace exist.s
	 */
	private boolean namespaceExist(final List < NameSpaceClass > lst,
			final String name) {
		for (int i = 0; i < lst.size(); i++) {
			NameSpaceClass nms = lst.get(i);
			if (nms.getNameSpaceName().compareTo(name) == 0) {
				return true;
			}
		}
		return false;
	}

	/**
	 * @return the nameJar
	 */
	public final String getNameJar() {
		String [] tmp  = nameJar.split("\\.");
		return tmp[0];
	}

	/**
	 * @param nameJarP the nameJar to set
	 */
	public final void setNameJar(final String nameJarP) {
		this.nameJar = nameJarP;
	}
}
