package com.bull.cimero.pluginEditor.utils.classLoader;

import java.lang.reflect.Constructor;
import java.lang.reflect.Method;

import com.bull.cimero.pluginEditor.utils.log.CimeroLog;

/**
 * This static class is used to call with a class loader a function.
 * @author Cedric Mouilleron
 * @author Pierre Notel
 * @author Arnaud Pons
 * @author Loic Albertin
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public final class CimeroClassLoader {

	/**
	 * private constructor.
	 *
	 */
	private CimeroClassLoader() {
		super();
	}

	/**
	 * @author Arnaud Pons
	 * @author Loc Albertin
	 * Instantiate an object and call a method on it, and finally returns the method's call result.
	 * @param className The object's name
	 * @param initParams Object's constructor parameters
	 * @param methodName The method's name
	 * @param params Method's parameters
	 * @return Returns the method's call result
	 * @throws Exception java Reflective api Exception can be throws
	 */
	public static Object callMethod(final String className,
			final Object [] initParams,
			final String methodName,
			final Object [] params) throws Exception {

		Object object = newInstance(className, initParams);

		return callMethod(object, methodName, params);
	}

	/**
	 * @author Arnaud Pons
	 * @author Loc Albertin
	 * Instantiate an object.
	 * @param className The object's name
	 * @param initParams Object's constructor parameters
	 * @return Returns the object
	 * @throws Exception java Reflective api Exception can be throws
	 */
	public static Object newInstance(final String className,
			final Object [] initParams) throws Exception {
		Class [] initParamsClasses = new Class[initParams.length];
		for (int i = 0; i < initParams.length; i++) {
			initParamsClasses[i] = initParams[i].getClass();
		}

		Class clazz = Class.forName(className);
		Constructor constructor = clazz.getConstructor(initParamsClasses);
		Object object = constructor.newInstance(initParams);

		return object;
	}
	
	
	public static Constructor getConstructorWithParameters(final String className,
			final Class [] constructorParams) throws ClassNotFoundException {
		
		try{
			Class clazz = Class.forName(className);
			Constructor constructor = clazz.getConstructor(constructorParams);
			return constructor;
		}catch(ClassNotFoundException cnfe){
			CimeroLog.errorLog(" Class Not Found ", cnfe);
			throw cnfe;
		}catch(NoSuchMethodException noSuchMethodException){
			CimeroLog.warningLog(" Constructor not found ", noSuchMethodException);
			return null;
		}

	}


	/**
	 * call a method with an object.
	 * @param obj : the object that contains the method
	 * @param methodName : the name of the method.
	 * @param params : the parameters of this method.
	 * @return the result method
	 * @throws Exception if the method doesn't exist.
	 */
	public static Object callMethod(final Object obj,
			final String methodName,
			final Object [] params) throws Exception {
		Class [] paramsClasses = new Class[params.length];
		for (int i = 0; i < params.length; i++) {
			paramsClasses[i] = params[i].getClass();
		}
		Class clazz = obj.getClass();

		Method method = clazz.getMethod(methodName, paramsClasses);

		return method.invoke(obj, params);
	}

}
