package com.bull.cimero.pluginEditor.utils.zip;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.zip.Adler32;
import java.util.zip.CheckedOutputStream;
import java.util.zip.Deflater;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;
import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.Path;
import com.bull.cimero.pluginEditor.utils.exception.CimeroGenerateJarException;
import com.bull.cimero.pluginEditor.utils.log.CimeroLog;

/**
 * class used to manage zip files.
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello ( andrea.zoppello@eng.it )
 */
public class ZipUtils {
	/** buffer size. */
	public static final int BUFFER = 2048;
	/** current ZipUtils instance. */
	private static ZipUtils zipUtils = null;

	/**
	 * method used to get the current instance of the class.
	 * @return ZipUtils : the instance
	 */
	public static ZipUtils getDefault() {
		if (zipUtils == null) {
			zipUtils = new ZipUtils();
		}
		return zipUtils;
	}

	/**
	 * zip method for a directory and its sub-folders.
	 * @param folder : folder where found files and sub-folders to zip
	 * @return boolean : true if no errors
	 * @throws CimeroGenerateJarException if error
	 */
	public final boolean zipFolderAndSubFolder(final IFolder folder)
				throws CimeroGenerateJarException {
		IResource[] tabRessource = null;
		try {
			tabRessource = folder.members();
		} catch (CoreException e) {
			throw new CimeroGenerateJarException(
					"unable to get files in a directory : "
					+ folder.getName(), e);
		}
		//for each resource
		for (int i = 0; i < tabRessource.length; i++) {
			//for each folder
			if (tabRessource[i].getType() == IResource.FOLDER) {
				//not for the META-INF directory
				if (!((IFolder) tabRessource[i]).getFullPath().toString()
						.contains("META-INF")) {
					zip((IFolder) tabRessource[i]);
					//we delete the folder
					try {
						((IFolder) tabRessource[i]).delete(true, null);
					} catch (CoreException e) {
						throw new CimeroGenerateJarException(
								"unable to delete a file : "
								+ ((IFolder) tabRessource[i]).getName(), e);
					}
				}
			}
		}
		//now, we can zip the main folder
		try {
			//refresh to apply modifications
			folder.refreshLocal(1, null);
			zip(folder);
			folder.delete(true, null);
			//refresh to apply modifications
			folder.refreshLocal(1, null);
		} catch (CoreException e) {
			throw new CimeroGenerateJarException(
					"unable to delete a folder : "
					+ folder.getName(), e);
		}
		return true;
	}

	/**
	 * zip method for a directory.
	 * @param folder : folder where found files to zip
	 * @return IFile : zipped file or null if error
	 * @throws CimeroGenerateJarException if error
	 */
	private IFile zip(final IFolder folder)
					throws CimeroGenerateJarException {
		IResource[] tabRessource = null;
		int count;
		try {
			tabRessource = folder.members();
		} catch (CoreException e) {
			throw new CimeroGenerateJarException(
					"unable to get files in a directory : "
					+ folder.getName(), e);
		} //stream creation
		FileOutputStream dest = null;
		//set the destination path
		String destination = folder.getName() + ".zip";
		//create the destination file
		IFile iFile = folder.getParent().getFile(new Path(destination));
		try {
			dest = new FileOutputStream(iFile.getLocation().toString());
		} catch (FileNotFoundException e) {
				throw new CimeroGenerateJarException(
					"unable to create destination file : "
					+ iFile.getName(), e);
		} //checksum: Adler32 (faster) or CRC32
		CheckedOutputStream checksum =
						new CheckedOutputStream(dest, new Adler32());
	  	//writer buffer creation
		BufferedOutputStream buff = new BufferedOutputStream(checksum);
	  	//zip writer creation
		ZipOutputStream out = new ZipOutputStream(buff);
	    //compression method
		out.setMethod(ZipOutputStream.DEFLATED);
		//compression quality
		out.setLevel(Deflater.BEST_COMPRESSION);
	    //temporary buffer
		byte[] data = new byte[BUFFER];
		//for each file in the list
		for (int i = 0; i < tabRessource.length; i++) {
			if (tabRessource[i].getType() == IResource.FILE) {
		    	//reading stream creation
		        FileInputStream fi = null;
				try {
					fi = new FileInputStream(
							((IFile) tabRessource[i]).getLocation().toString());
				} catch (FileNotFoundException e) {
					throw new CimeroGenerateJarException(
							"unable to find a file : "
							+  ((IFile) tabRessource[i]).getLocation()
									.toString(), e);
				} //reading buffer creation
				BufferedInputStream buffi = new BufferedInputStream(fi, BUFFER);
				//add a zip entry
				ZipEntry entry = new ZipEntry(
						((IFile) tabRessource[i]).getName());
				//add to the zip
				try {
					out.putNextEntry(entry);
				} catch (IOException e) {
					throw new CimeroGenerateJarException("unable to put an entr"
							+ "y in the zip file : " +  entry.getName(), e);
				} //writing to the writer stream
				try {
					while ((count = buffi.read(data, 0, BUFFER)) != -1) {
						out.write(data, 0, count);
					}
				} catch (IOException e) {
					throw new CimeroGenerateJarException(
							"unable to put an entry in the zip file : "
							+  entry.getName(), e);
				} //close the current entry
				try {
					out.closeEntry();
					buffi.close();
				} catch (IOException e) {
					CimeroLog.warningLog(
							"Error when closing a file writer stream ! "
							+ "Try to continue !", e);
				}
			} //end if it's a file
			if (tabRessource[i].getType() == IResource.FOLDER) {
				//it's a folder --> META-INF
				IResource[] tabRessourceFolder = null;
				try {
					tabRessourceFolder = ((IFolder) tabRessource[i]).members();
				} catch (CoreException e) {
					throw new CimeroGenerateJarException(
							"unable to get files in a directory : "
							+  ((IFolder) tabRessource[i]).getName(), e);
				} //for each file in this new list
				for (int j = 0; j < tabRessourceFolder.length; j++) {
					//reading stream creation
			        FileInputStream fi = null;
					try {
						fi = new FileInputStream(((IFile) tabRessourceFolder[j])
													.getLocation().toString());
					} catch (FileNotFoundException e) {
						throw new CimeroGenerateJarException(
								"unable to find a file : "
								+  ((IFile) tabRessourceFolder[j])
									.getLocation().toString(), e);
					}
					//reading buffer creation
					BufferedInputStream buffi = new BufferedInputStream(
											fi, BUFFER);
					//add a zip entry
					//split the location on the file.separator
					String[] tabStr = ((String) ((IFile) tabRessourceFolder[j])
							.getLocation().toString())
							.split("/");
					ZipEntry entry = new ZipEntry(
							tabStr[tabStr.length - 2] + "/"
							+ ((IFile) tabRessourceFolder[j]).getName());
					//add to the zip
					try {
						out.putNextEntry(entry);
					} catch (IOException e) {
						throw new CimeroGenerateJarException(
								"unable to add entry in the zip file : "
								+  entry.getName(), e);
					} //writing to the writer stream
					try {
						while ((count = buffi.read(data, 0, BUFFER)) != -1) {
							out.write(data, 0, count);
						}
					} catch (IOException e) {
						throw new CimeroGenerateJarException(
								"unable to add entry in the zip file : "
								+  entry.getName(), e);
					} //close the current entry
					try {
						out.closeEntry();
						buffi.close();
					} catch (IOException e) {
						CimeroLog.warningLog(
								"Error when closing a file writer stream ! "
								+ "Try to continue !", e);
					}
				}
			}
		}
		//stream closed
		try {
			out.close();
			buff.close();
			checksum.close();
			dest.close();
		} catch (IOException e) {
			CimeroLog.warningLog(
					"Error when closing streams ! Try to continue !", e);
		} //no fatal error
		return iFile;
	}

	/**
	 * create a directory in the current namespace. The name of this directory
	 * is pass in parameter.
	 * @param string : the name of the new directory.
	 * @param parent : the parent where this directory must to be add
	 * @return null if there is an error and the IFolder if ok
	 */
	public final IFolder createDirectory(final String string,
			final IContainer parent) {
		IFolder targetFolder = parent.getFolder(new Path(string));
		if (targetFolder.exists()) {
			return null;
		}
		try {
			targetFolder.create(true, false, null);
		} catch (CoreException e) {
			CimeroLog.errorLog("exception during directory creation", e);
		}
		return targetFolder;
	}

	/**
	 * delete the directory that has the name in parameter.
	 * Delete all the file of this directory.
	 * @param parent the container where this directory is situated
	 * @param dirName : name of the directory to delete
	 */
	public final void deleteDirectory(final String dirName,
				final IContainer parent) {
		IFolder targetFolder = parent.getFolder(new Path(dirName));
		if (targetFolder.exists()) {
			try {
				targetFolder.delete(true, null);
			} catch (CoreException e) {
				return;
			}
		}
	}
}
