package com.bull.cimero.pluginEditor.wizards;

import java.io.File;
import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.dialogs.ContainerSelectionDialog;
import cimeroEditor.CimeroEditorPlugin;

/**
 * The "New" wizard page allows setting the container for the new file as well
 * as the file name. The page will only accept file name without the extension
 * OR with the extension that matches the expected one (cimero).
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello ( andrea.zoppello@eng.it )
 */
public class CimeroNewWizardPage extends WizardPage {

	/** container. */
	private Text containerText;
	/** text. */
	private Text fileText;
	/** selection. */
	private ISelection selection;
	/** checkbutton to complex sample. */

	/** number of columns. */
	private static final int NUMBEROFCOLUMNS = 3;
	/** vertical spacing. */
	private static final int VERTICALSPACING = 9;
	/** minimum width. */
	private static final int MINIMUMWIDTH = 350;

	/**
	 * Constructor for SampleNewWizardPage.
	 * @param selectionParam : current selection
	 */
	public CimeroNewWizardPage(final ISelection selectionParam) {
		//tell the parent constructor and give it the image to print
		super("wizardPage" , "Create a New Integration Process",
				CimeroEditorPlugin.getImageDescriptor(
						"icons" + File.separator
						+ "spagic_logo.jpg"));
		//description
		setDescription("This wizard creates a new file with "
				+ "*.spagic extension that can be opened by "
				+ "the Spagic editor.");
		this.selection = selectionParam;
	}

	/**
	 * @see IDialogPage#createControl(Composite)
	 * @param parent : parent composite
	 */
	public final void createControl(final Composite parent) {
		Composite container = new Composite(parent, SWT.NULL);
		RowLayout layout = new RowLayout();
		container.setLayout(layout);
		layout.type = SWT.VERTICAL;
		//
		GridLayout topLayout = new GridLayout();
		topLayout.numColumns = NUMBEROFCOLUMNS;
		topLayout.verticalSpacing = VERTICALSPACING;
		Composite topContainer = new Composite(container, SWT.NULL);
		topContainer.setLayout(topLayout);

		Label label = new Label(topContainer, SWT.NULL);
		label.setText("&Container:");

		containerText = new Text(topContainer, SWT.BORDER | SWT.SINGLE);
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		containerText.setLayoutData(gd);

		containerText.addModifyListener(new ModifyListener() {
			public void modifyText(final ModifyEvent e) {
				dialogChanged();
			}
		});

		Button button = new Button(topContainer, SWT.PUSH);
		button.setText("Browse...");
		button.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(final SelectionEvent e) {
				handleBrowse();
			}
		});
		label = new Label(topContainer, SWT.NULL);
		label.setText("&File name:");

		fileText = new Text(topContainer, SWT.BORDER | SWT.SINGLE);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.minimumWidth = MINIMUMWIDTH;
		fileText.setLayoutData(gd);
		fileText.addModifyListener(new ModifyListener() {
			public void modifyText(final ModifyEvent e) {
				dialogChanged();
			}
		});
		/*
		//add of the container with advanced options
		GridLayout bottomLayout = new GridLayout();
		bottomLayout.numColumns = 1;
		bottomLayout.verticalSpacing = VERTICALSPACING;
		Composite bottomContainer = new Composite(container, SWT.NULL);
		bottomContainer.setLayout(bottomLayout);
		//add the Basic sample
		checkBasicSample = new Button(bottomContainer, SWT.CHECK);
		checkBasicSample.setText("Add the Basic ServiceMix sample. ");
		checkBasicSample.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(final SelectionEvent e) {
				checkBasicSampleEvent();
			}
		});
		//add the RSS Binding sample
		checkRSSBindingSample = new Button(bottomContainer, SWT.CHECK);
		checkRSSBindingSample.setText("Add the RSS Binding ServiceMix sample.");
		checkRSSBindingSample.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(final SelectionEvent e) {
				checkRSSBindingSampleEvent();
			}
		});
		//add the JMS Binding sample
		checkJMSBindingSample = new Button(bottomContainer, SWT.CHECK);
		checkJMSBindingSample.setText("Add the JMS Binding ServiceMix sample.");
		checkJMSBindingSample.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(final SelectionEvent e) {
				checkJMSBindingSampleEvent();
			}
		});
		//add the Quartz Binding sample
		checkQuartzBindingSample = new Button(bottomContainer, SWT.CHECK);
		checkQuartzBindingSample.setText(
				"Add the Quartz Binding ServiceMix sample.");
		checkQuartzBindingSample.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(final SelectionEvent e) {
				checkQuartzBindingSampleEvent();
			}
		});
		//add the complex sample
		checkComplexSample = new Button(bottomContainer, SWT.CHECK);
		checkComplexSample.setText("Add a CIMERO sample in the new file. \n"
				+ "(you have to configure some of the components)");
		checkComplexSample.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(final SelectionEvent e) {
				checkComplexSampleEvent();
			}
		});
		//
		 * 
		 */
		initialize();
		dialogChanged();
		setControl(container);
	}

	

	

	

	

	

	/**
	 * Tests if the current workbench selection is a suitable container to use.
	 */
	private void initialize() {
		if (selection != null && !selection.isEmpty()
				&& selection instanceof IStructuredSelection) {
			IStructuredSelection ssel = (IStructuredSelection) selection;
			if (ssel.size() > 1) {
				return;
			}
			Object obj = ssel.getFirstElement();
			if (obj instanceof IResource) {
				IContainer container;
				if (obj instanceof IContainer) {
					container = (IContainer) obj;
				} else {
					container = ((IResource) obj).getParent();
				}
				containerText.setText(container.getFullPath().toString());
			}
		}
		fileText.setText("default.spagic");
	}

	/**
	 * Uses the standard container selection dialog to choose the new value for
	 * the container field.
	 */
	private void handleBrowse() {
		ContainerSelectionDialog dialog = new ContainerSelectionDialog(
				getShell(), ResourcesPlugin.getWorkspace().getRoot(), false,
				"Select new file container");
		if (dialog.open() == ContainerSelectionDialog.OK) {
			Object[] result = dialog.getResult();
			if (result.length == 1) {
				containerText.setText(((Path) result[0]).toString());
			}
		}
	}

	/**
	 * Ensures that both text fields are set.
	 */
	private void dialogChanged() {
		IResource container = ResourcesPlugin.getWorkspace().getRoot()
				.findMember(new Path(getContainerName()));
		String fileName = getFileName();

		if (getContainerName().length() == 0) {
			updateStatus("File container must be specified");
			return;
		}
		if (container == null
				|| (container.getType()
						& (IResource.PROJECT | IResource.FOLDER)) == 0) {
			updateStatus("File container must exist");
			return;
		}
		if (!container.isAccessible()) {
			updateStatus("Project must be writable");
			return;
		}
		if (fileName.length() == 0) {
			updateStatus("File name must be specified");
			return;
		}
		if (fileName.replace('\\', '/').indexOf('/', 1) > 0) {
			updateStatus("File name must be valid");
			return;
		}
		int dotLoc = fileName.lastIndexOf('.');
		if (dotLoc != -1) {
			String ext = fileName.substring(dotLoc + 1);
			if (!ext.equalsIgnoreCase("spagic")) {
				updateStatus("File extension must be \"spagic\"");
				return;
			}
		}
		updateStatus(null);
	}

	/**
	 * change status wizard.
	 * @param message : message to status
	 */
	private void updateStatus(final String message) {
		setErrorMessage(message);
		setPageComplete(message == null);
	}

	/**
	 * get the name of the container.
	 * @return String
	 */
	public final String getContainerName() {
		return containerText.getText();
	}

	/**
	 * get the file name.
	 * @return String
	 */
	public final String getFileName() {
		return fileText.getText();
	}

	

	

	

	
	
}
